@@ -1,25 +1,32 @@
-;==============================================================================
+;;;============================================================================
 
-; file: "_source.scm"
+;;; File: "_source.scm", Time-stamp: <2007-11-20 22:14:12 feeley>
 
-; Copyright (C) 1994-1998 by Marc Feeley, All Rights Reserved.
+;;; Copyright (c) 1994-2007 by Marc Feeley, All Rights Reserved.
 
 (include "fixnum.scm")
 
-;------------------------------------------------------------------------------
-;
-; Source code manipulation module:
-; -------------------------------
+(include-adt "_envadt.scm")
+(include-adt "_gvmadt.scm")
+(include-adt "_ptreeadt.scm")
+(include     "_sourceadt.scm")
 
-; This module contains procedures to manipulate source code
-; representations read in from Scheme source files.
+'(##include "_hostadt.scm");*******************brad
 
-; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+;;;----------------------------------------------------------------------------
+;;
+;; Source code manipulation module:
+;; -------------------------------
 
-; A readenv structure maintains the "read environment" throughout the
-; reading of a Scheme datum.  It includes the port from which to read,
-; the readtable, the wrap and unwrap procedures, and the position
-; where the currently being read datum started.
+;; This module contains procedures to manipulate source code
+;; representations read in from Scheme source files.
+
+;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+
+;; A readenv structure maintains the "read environment" throughout the
+;; reading of a Scheme datum.  It includes the port from which to read,
+;; the readtable, the wrap and unwrap procedures, and the position
+;; where the currently being read datum started.
 
 (define (**make-readenv port readtable error-proc wrapper unwrapper)
   (vector port readtable error-proc wrapper unwrapper 0 0 0 0))
@@ -133,9 +140,9 @@
                   (loop2 i (cdr y)))))
             result))))))
 
-; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-;
-; Symbol "canonicalization".
+;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+;;
+;; Symbol "canonicalization".
 
 (define (string->canonical-symbol str)
   (let ((new-str (string-append str "")))
@@ -143,9 +150,9 @@
       (**readtable-string-convert-case! **main-readtable new-str))
     (string->symbol new-str)))
 
-; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-;
-; 'location' manipulation
+;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+;;
+;; 'location' manipulation
 
 (define (re->locat re filename)
   (vector filename (**readenv-filepos re)))
@@ -168,7 +175,7 @@
                 (col (+ (**filepos-col filepos) 1))
                 (filename*
                  (if (string? filename)
-                   (path-expand filename 'shortest)
+                   (path-expand filename)
                    filename)))
             (display prefix)
             (write filename*)
@@ -186,47 +193,39 @@
 
     (display "UNKNOWN LOCATION")))
 
-(define (locat-filename loc)
+(define (locat-filename-and-line loc)
   (if loc
-    (if (string? (vector-ref loc 0))
-      (vector-ref loc 0)
-      (let ((source (vector-ref loc 0)))
-        (if source
-          (locat-filename (source-locat source))
-          "")))
-    ""))
+    (if (string? (vector-ref loc 0)) ; file?
+      (let* ((filename (vector-ref loc 0))
+             (filepos (vector-ref loc 1))
+             (line (+ (**filepos-line filepos) 1)))
+        (cons filename line))
+      (let ((source (vector-ref loc 0))
+            (expr (vector-ref loc 1)))
+       (if source
+         (locat-filename-and-line (source-locat source))
+         (cons "" 1))))
+    (cons "" 1)))
 
-; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-;
-; 'source' manipulation
+(define (locat-filename loc)
+  (car (locat-filename-and-line loc)))
 
-(define (make-source code locat)
-  (vector code
-          (vector-ref locat 0)
-          (vector-ref locat 1)))
+;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+;;
+;; 'source' manipulation
 
-(define (source-code x)
-  (vector-ref x 0))
-
-(define (source-locat x)
-  (vector (vector-ref x 1)
-          (vector-ref x 2)))
-
-; (expression->source expr source) returns the source that represent the Scheme
-; expression 'expr' and is related to the source 'source' (#f if no relation).
+;; (expression->source expr source) returns the source that represent
+;; the Scheme expression 'expr' and is related to the source 'source'
+;; (#f if no relation).
 
 (define (expression->source expr source)
 
   (define (expr->source x)
     (make-source (cond ((pair? x)
                         (list-convert x))
-                       ((or (u8vect? x)
-                            (u16vect? x)
-                            (u32vect? x)
-                            (f32vect? x)
-                            (f64vect? x))
-                        x)
-                       ((vector? x)
+                       ((box-object? x)
+                        (box-object (expr->source (unbox-object x))))
+                       ((vector-object? x)
                         (vector-convert x))
                        (else
                         x))
@@ -269,9 +268,9 @@
 
   (expr->source expr))
 
-; (source->expression source) returns the Scheme expression represented by the
-; source 'source'.  Note that every call with the same argument returns a
-; different (i.e. non eq?) expression.
+;; (source->expression source) returns the Scheme expression represented by the
+;; source 'source'.  Note that every call with the same argument returns a
+;; different (i.e. non eq?) expression.
 
 (define (source->expression source)
 
@@ -296,24 +295,20 @@
   (let ((code (source-code source)))
     (cond ((pair? code)
            (list->expression code))
-          ((or (u8vect? code)
-               (u16vect? code)
-               (u32vect? code)
-               (f32vect? code)
-               (f64vect? code))
-           code)
-          ((vector? code)
+          ((box-object? code)
+           (box-object (source->expression (unbox-object code))))
+          ((vector-object? code)
            (vector->expression code))
           (else
            code))))
 
-; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-; (include-expr->sources source info-port) returns a list of the
-; source representation for each of the expressions contained in the
-; specified file.
+;; (include-expr->source source info-port) returns a list of the
+;; source representation for each of the expressions contained in the
+;; specified file.
 
-(define (include-expr->sources source info-port)
+(define (include-expr->sourcezzzzz source info-port)
 
   (define (find-source-file filename)
 
@@ -321,10 +316,10 @@
       (pt-syntax-error
        source
        "Can't find file"
-       (or (path-expand filename 'shortest)
+       (or (path-expand filename)
            filename)))
 
-    (let ((expanded-filename (path-expand filename 'absolute)))
+    (let ((expanded-filename (path-expand filename)))
       (if expanded-filename
         (if (equal? (path-extension expanded-filename) "")
 
@@ -354,28 +349,40 @@
          (filename
           (source-code filename-src))
          (rerooted-filename
-          (if (path-absolute? filename)
-            filename
-            (string-append
-             (path-directory (locat-filename (source-locat filename-src)))
-             filename)))
+          (path-expand
+           filename
+           (path-directory (path-expand (locat-filename (source-locat filename-src))))))
          (final-filename
           (find-source-file rerooted-filename))
          (re
           (**readenv-open final-filename)))
 
     (define (read-sources) ; return list of all sources in file
+';;;;;;;;;;;;;;;;;;;;;;;;;;;
       (let ((source ((or read-datum-or-eof **read-datum-or-eof) re)))
-        (if (vector? source)
+        (if (vector-object? source)
           (begin
             (if info-port (display "." info-port))
             (cons source (read-sources)))
-          '())))
+          '()))
+      (##vector-ref
+       (##read-all-as-a-begin-expr-from-port
+        (**readenv-port re)
+        (##current-readtable)
+        (lambda (re x)
+          (make-source x
+                       (##make-locat (##port-name (macro-readenv-port re))
+                                     (macro-readenv-filepos re))))
+        (lambda (re x)
+          (source-code x))
+        #f
+        #f)
+       1));;;;;;;;;;;;;;;;;;;;;;;
 
     (if info-port
       (begin
         (display "(reading " info-port)
-        (write (path-expand final-filename 'shortest) info-port)))
+        (write (path-expand final-filename) info-port)))
 
     (let ((sources (read-sources)))
 
@@ -385,9 +392,54 @@
 
       sources)))
 
-; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+;; Filename extensions to try in order to find source files from module name.
 
-; Tables for reader.
+(define scm-file-exts '(".scm" ".six" "")) ; "" means no extension
+
+
+(define (read-source path relative-to-path try-scheme-file-extensions?)
+
+  (define (read-source-from-path path)
+    (##read-all-as-a-begin-expr-from-path
+     path
+     (##current-readtable);;;;;;;;;;;;;;;;;;;;
+     (lambda (re x)
+       (make-source x (##make-locat-from-readenv re)))
+     (lambda (re x)
+       (source-code x))))
+
+  (define (read-source-no-extension)
+    (let loop ((lst ##scheme-file-extensions))
+      (if (pair? lst)
+        (let ((x (read-source-from-path (string-append path (caar lst)))))
+          (if (##fixnum? x)
+            (loop (cdr lst))
+            x))
+        #f)))
+
+  (or (and try-scheme-file-extensions?
+           (string=? (path-extension path) "")
+           (read-source-no-extension))
+      (let* ((abs-path (##path-reference path relative-to-path))
+             (x (read-source-from-path abs-path)))
+        (if (##fixnum? x)
+          (compiler-error "Can't find file" abs-path)
+          x))))
+
+(define (include-expr->source source info-port)
+  (let* ((filename-src
+          (cadr (source-code source)))
+         (filename
+          (source-code filename-src))
+         (x
+          (read-source filename
+                       (locat-filename (source-locat filename-src))
+                       #f)))
+    (##vector-ref x 1)));;;;;;;;;;;;;;;;;;;;;;;;
+
+;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+
+;; Tables for reader.
 
 (define **standard-escaped-char-table
   (list
@@ -419,21 +471,26 @@
     (cons "optional" optional-object)
     (cons "rest"     rest-object)
     (cons "key"      key-object)
-    (cons "eof"      end-of-file-object)
-    (cons ""         script-object)))
+    (cons "eof"      end-of-file-object)))
 
-;==============================================================================
+(set! **standard-sharp-bang-table;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+      (append (list (cons "void"     (##type-cast -5 2))
+                    (cons "unbound1" (##type-cast -7 2))
+                    (cons "unbound2" (##type-cast -8 2)))
+                **standard-sharp-bang-table))
 
-; The reader.
+;;;============================================================================
 
-; For compatibility between the interpreter and compiler, this section
-; must be the same as the corresponding section in the file
-; "lib/_io.scm" (except that ## and ** are exchanged).
+;; The reader.
 
-; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+;; For compatibility between the interpreter and compiler, this section
+;; must be the same as the corresponding section in the file
+;; "lib/_io.scm" (except that ## and ** are exchanged).
 
-; A chartable structure is a vector-like data structure which is
-; indexed using a character.
+;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+
+;; A chartable structure is a vector-like data structure which is
+;; indexed using a character.
 
 (define (**make-chartable default)
   (vector (make-vector 128 default) default '()))
@@ -456,11 +513,11 @@
           (set-cdr! x val)
           (vector-set! ct 2 (cons (cons i val) (vector-ref ct 2))))))))
 
-; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-; A readtable structure contains parsing information for the reader.
-; It indicates what action must be taken when a given character is
-; encountered.
+;; A readtable structure contains parsing information for the reader.
+;; It indicates what action must be taken when a given character is
+;; encountered.
 
 (define **readtable-tag '#(readtable 0))
 
@@ -472,7 +529,7 @@
          sharp-bang-table
          char-delimiter?-table
          char-handler-table)
-  (**subtype-set!
+;;  (**subtype-set!
     (vector
      **readtable-tag
      case-conversion?
@@ -482,7 +539,8 @@
      sharp-bang-table
      char-delimiter?-table
      char-handler-table)
-    (subtype-structure)))
+;;    (subtype-structure))
+)
 
 (define (**readtable-case-conversion? rt)
   (vector-ref rt 1))
@@ -579,9 +637,9 @@
                          (string->keyword-object
                           (substring s 0 (- len 1))))))))))
 
-; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-; Error handling.
+;; Error handling.
 
 (define (**read-error-datum-or-eof-expected re)
   ((**readenv-error-proc re) re "Datum or EOF expected"))
@@ -613,6 +671,9 @@
 (define (**read-error-u32 re)
   ((**readenv-error-proc re) re "32 bit exact integer expected"))
 
+(define (**read-error-u64 re)
+  ((**readenv-error-proc re) re "64 bit exact integer expected"))
+
 (define (**read-error-f32/f64 re)
   ((**readenv-error-proc re) re "Inexact real expected"))
 
@@ -634,9 +695,9 @@
 (define (**read-error-char-range re)
   ((**readenv-error-proc re) re "Character out of range"))
 
-; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-; Procedures to read single characters.
+;; Procedures to read single characters.
 
 (define (**peek-next-char re) ; never returns end-of-file
   (let ((next (**peek-next-char-or-eof re)))
@@ -658,19 +719,19 @@
       (**read-error-incomplete-form-eof-reached re))
     x))
 
-; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-; Procedures to read datums.
+;; Procedures to read datums.
 
-; (**read-datum-or-eof re) attempts to read a datum in the read
-; environment "re", skipping all whitespace and comments in the
-; process.  The "pos" field of the read environment indicates the
-; position where the enclosing datum starts (e.g. list or vector).  If
-; a datum is read it is returned (wrapped if the read environment asks
-; for it); if the end-of-file is reached the end-of-file object is
-; returned (never wrapped); otherwise an error is signaled.  The read
-; environment's "pos" field is only modified if a datum was read, in
-; which case it is the position where the datum starts.
+;; (**read-datum-or-eof re) attempts to read a datum in the read
+;; environment "re", skipping all whitespace and comments in the
+;; process.  The "pos" field of the read environment indicates the
+;; position where the enclosing datum starts (e.g. list or vector).  If
+;; a datum is read it is returned (wrapped if the read environment asks
+;; for it); if the end-of-file is reached the end-of-file object is
+;; returned (never wrapped); otherwise an error is signaled.  The read
+;; environment's "pos" field is only modified if a datum was read, in
+;; which case it is the position where the datum starts.
 
 (define (**read-datum-or-eof re)
   (let ((obj (**read-datum-or-none re)))
@@ -686,14 +747,14 @@
             c))) ; end-of-file was reached so return end-of-file object
       obj)))
 
-; (**read-datum re) attempts to read a datum in the read environment
-; "re", skipping all whitespace and comments in the process.  The
-; "pos" field of the read environment indicates the position where the
-; enclosing datum starts (e.g. list or vector).  If a datum is read it
-; is returned (wrapped if the read environment asks for it); if the
-; end-of-file is reached or no datum can be read an error is signaled.
-; The read environment's "pos" field is only modified if a datum was
-; read, in which case it is the position where the datum starts.
+;; (**read-datum re) attempts to read a datum in the read environment
+;; "re", skipping all whitespace and comments in the process.  The
+;; "pos" field of the read environment indicates the position where the
+;; enclosing datum starts (e.g. list or vector).  If a datum is read it
+;; is returned (wrapped if the read environment asks for it); if the
+;; end-of-file is reached or no datum can be read an error is signaled.
+;; The read environment's "pos" field is only modified if a datum was
+;; read, in which case it is the position where the datum starts.
 
 (define (**read-datum re)
   (let ((obj (**read-datum-or-none re)))
@@ -704,15 +765,15 @@
         (**read-error-datum-expected re))
       obj)))
 
-; (**read-datum-or-none re) attempts to read a datum in the read
-; environment "re", skipping all whitespace and comments in the
-; process.  The "pos" field of the read environment indicates the
-; position where the enclosing datum starts (e.g. list or vector).  If
-; a datum is read it is returned (wrapped if the read environment asks
-; for it); if the end-of-file is reached or no datum can be read the
-; "none-marker" is returned.  The read environment's "pos" field is
-; only modified if a datum was read, in which case it is the position
-; where the datum starts.
+;; (**read-datum-or-none re) attempts to read a datum in the read
+;; environment "re", skipping all whitespace and comments in the
+;; process.  The "pos" field of the read environment indicates the
+;; position where the enclosing datum starts (e.g. list or vector).  If
+;; a datum is read it is returned (wrapped if the read environment asks
+;; for it); if the end-of-file is reached or no datum can be read the
+;; "none-marker" is returned.  The read environment's "pos" field is
+;; only modified if a datum was read, in which case it is the position
+;; where the datum starts.
 
 (define (**read-datum-or-none re)
   (let ((obj (**read-datum-or-none-or-dot re)))
@@ -722,16 +783,16 @@
         (**read-error-improperly-placed-dot re))
       obj)))
 
-; (**read-datum-or-none-or-dot re) attempts to read a datum in the
-; read environment "re", skipping all whitespace and comments in the
-; process.  The "pos" field of the read environment indicates the
-; position where the enclosing datum starts (e.g. list or vector).  If
-; a datum is read it is returned (wrapped if the read environment asks
-; for it); if a lone dot is read the "dot-marker" is returned; if the
-; end-of-file is reached or no datum can be read the "none-marker" is
-; returned.  The read environment's "pos" field is only modified if a
-; datum was read, in which case it is the position where the datum
-; starts.
+;; (**read-datum-or-none-or-dot re) attempts to read a datum in the
+;; read environment "re", skipping all whitespace and comments in the
+;; process.  The "pos" field of the read environment indicates the
+;; position where the enclosing datum starts (e.g. list or vector).  If
+;; a datum is read it is returned (wrapped if the read environment asks
+;; for it); if a lone dot is read the "dot-marker" is returned; if the
+;; end-of-file is reached or no datum can be read the "none-marker" is
+;; returned.  The read environment's "pos" field is only modified if a
+;; datum was read, in which case it is the position where the datum
+;; starts.
 
 (define (**read-datum-or-none-or-dot re)
   (let ((next (**peek-next-char-or-eof re)))
@@ -739,14 +800,14 @@
       ((**readtable-char-handler (**readenv-readtable re) next) re next)
       (**none-marker))))
 
-; Special objects returned by **read-datum-or-none-or-dot.
+;; Special objects returned by **read-datum-or-none-or-dot.
 
 (define (**none-marker) '#(none)) ; indicates no following datum
 (define (**dot-marker) '#(dot))   ; indicates an isolated dot
 
-; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-; Procedure to read a list of datums (possibly an improper list).
+;; Procedure to read a list of datums (possibly an improper list).
 
 (define (**build-list re allow-improper? start-pos close)
   (let ((obj (**read-datum-or-none re)))
@@ -782,9 +843,9 @@
                      (set-cdr! end tail)
                      (loop tail))))))))))
 
-; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-; Procedure to read a vector or byte vector.
+;; Procedure to read a vector or byte vector.
 
 (define (**build-vector re kind start-pos close)
 
@@ -809,6 +870,7 @@
             ((u8vector)  (make-u8vect i))
             ((u16vector) (make-u16vect i))
             ((u32vector) (make-u32vect i))
+            ((u64vector) (make-u64vect i))
             ((f32vector) (make-f32vect i))
             ((f64vector) (make-f64vect i))))
         (let ((vect (loop (+ i 1))))
@@ -836,6 +898,13 @@
                    (**readenv-filepos-set! re x-pos) ; restore pos of element
                    (**read-error-u32 re)))
                (u32vect-set! vect i ux)))
+            ((u64vector)
+             (let ((ux (**readenv-unwrap re x)))
+               (if (not (exact-integer-check ux 0 18446744073709551615))
+                 (begin
+                   (**readenv-filepos-set! re x-pos) ; restore pos of element
+                   (**read-error-u64 re)))
+               (u64vect-set! vect i ux)))
             ((f32vector)
              (let ((ux (**readenv-unwrap re x)))
                (if (not (inexact-real-check ux))
@@ -852,9 +921,9 @@
                (f64vect-set! vect i ux))))
           vect)))))
 
-; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-; Procedures to read delimited tokens.
+;; Procedures to read delimited tokens.
 
 (define (**build-delimited-string re c i)
   (let loop ((i i))
@@ -881,7 +950,7 @@
     (**readtable-string-convert-case! (**readenv-readtable re) s)
     (string->symbol s)))
 
-; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
 (define (**build-escaped-string-up-to re close)
 
@@ -894,7 +963,7 @@
         (and (not (char<? c #\A)) (not (char<? #\F c)))))
 
   (define (unicode n)
-    (if (in-unicode-range? n)
+    (if (in-char-range? n)
       (unicode->character n)
       (**read-error-char-range re)))
 
@@ -978,9 +1047,9 @@
             (**append-strings (reverse new-chunks))
             (loop new-chunks)))))))
 
-; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-; Procedures to handle comments.
+;; Procedures to handle comments.
 
 (define (**skip-extended-comment re open1 open2 close1 close2)
   (let loop ((level 0) (c (**read-next-char re)))
@@ -1008,9 +1077,9 @@
           (if (not (char=? next #\newline))
             (loop)))))))
 
-; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-; Procedure to read datums starting with '#'.
+;; Procedure to read datums starting with '#'.
 
 (define (**read-sharp re c)
   (let ((start-pos (**readenv-current-filepos re)))
@@ -1040,7 +1109,7 @@
                          (if (and n
                                   (integer? n)
                                   (exact? n))
-                           (if (in-unicode-range? n)
+                           (if (in-char-range? n)
                              (**readenv-wrap re (unicode->character n))
                              (**read-error-char-range re))
                            (**read-error-char-name re name)))))))))
@@ -1091,6 +1160,8 @@
                                   (build-vect re 'u16vector))
                                  ((string-ci=? s "#u32")
                                   (build-vect re 'u32vector))
+                                 ((string-ci=? s "#u64")
+                                  (build-vect re 'u64vector))
                                  ((string-ci=? s "#f32")
                                   (build-vect re 'f32vector))
                                  ((string-ci=? s "#f64")
@@ -1099,7 +1170,7 @@
                                   (**read-error-sharp-token re s)))))))
                (**readenv-wrap re obj)))))))
 
-; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
 (define (**read-whitespace re c)
   (**read-next-char-or-eof re) ; skip whitespace character
@@ -1211,9 +1282,9 @@
             (loop (cdr lst)))))
       #f)))
 
-; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-; Setup the standard readtable.
+;; Setup the standard readtable.
 
 (define (**make-standard-readtable)
   (let ((rt
@@ -1282,4 +1353,260 @@
   (set! **main-readtable
     (**make-standard-readtable)))
 
-;==============================================================================
+;;;============================================================================
+
+'(;;;;;;;;;;;
+
+(include "fixnum.scm")
+
+(include-adt "_envadt.scm")
+(include-adt "_gvmadt.scm")
+(include-adt "_ptreeadt.scm")
+(include     "_sourceadt.scm")
+
+(define (**filepos-line filepos)
+  (##filepos-line filepos))
+
+;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+;;
+;; Symbol "canonicalization".
+
+(define (string->canonical-symbol str)
+  (let ((new-str (string-append str "")))
+    (##readtable-string-convert-case! (current-readtable) new-str)
+    new-str))
+
+;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+;;
+;; 'location' manipulation
+
+(define (expr->locat expr source)
+  (vector source expr));;;;;;;;;;;;;;;;;;;;;;
+
+(define (locat-show prefix loc)
+  (if loc
+
+    (if (string? (vector-ref loc 0)) ; file?
+      (let* ((filename (vector-ref loc 0))
+             (filepos (vector-ref loc 1))
+             (str (format-filepos filename filepos #t)))
+        (if str
+          (begin
+            (display prefix)
+            (display str))
+          (let ((line (+ (**filepos-line filepos) 1))
+                (col (+ (**filepos-col filepos) 1))
+                (filename*
+                 (if (string? filename)
+                   (path-expand filename)
+                   filename)))
+            (display prefix)
+            (write filename*)
+            (display "@")
+            (display line)
+            (display ".")
+            (display col))))
+      (let ((source (vector-ref loc 0))
+            (expr (vector-ref loc 1)))
+       (display prefix)
+       (display "EXPRESSION ")
+       (write expr)
+       (if source
+         (locat-show " " (source-locat source)))))
+
+    (display "UNKNOWN LOCATION")))
+
+(define (locat-filename-and-line loc)
+  (if loc
+    (if (string? (vector-ref loc 0)) ; file?
+      (let* ((filename (vector-ref loc 0))
+             (filepos (vector-ref loc 1))
+             (line (+ (**filepos-line filepos) 1)))
+        (cons filename line))
+      (let ((source (vector-ref loc 0))
+            (expr (vector-ref loc 1)))
+       (if source
+         (locat-filename-and-line (source-locat source))
+         (cons "" 1))))
+    (cons "" 1)))
+
+;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+;;
+;; 'source' manipulation
+
+;; (expression->source expr source) returns the source that represent
+;; the Scheme expression 'expr' and is related to the source 'source'
+;; (#f if no relation).
+
+(define (expression->source expr source)
+
+  (define (expr->source x)
+    (make-source (cond ((pair? x)
+                        (list-convert x))
+                       ((box-object? x)
+                        (box-object (expr->source (unbox-object x))))
+                       ((vector-object? x)
+                        (vector-convert x))
+                       (else
+                        x))
+                 (expr->locat x source)))
+
+  (define (list-convert l)
+    (cons (expr->source (car l))
+          (list-tail-convert (cdr l))))
+
+  (define (list-tail-convert l)
+    (cond ((pair? l)
+           (if (quoting-form? l) ; so that macros which generate quoting-forms
+             (expr->source l)    ; at the tail of a list work properly
+             (cons (expr->source (car l))
+                   (list-tail-convert (cdr l)))))
+          ((null? l)
+           '())
+          (else
+           (expr->source l))))
+
+  (define (quoting-form? x)
+    (let ((first (car x))
+          (rest (cdr x)))
+      (and (pair? rest)
+           (null? (cdr rest))
+           (or (eq? first quote-sym)
+               (eq? first quasiquote-sym)
+               (eq? first unquote-sym)
+               (eq? first unquote-splicing-sym)))))
+
+  (define (vector-convert v)
+    (let* ((len (vector-length v))
+           (x (make-vector len)))
+      (let loop ((i (- len 1)))
+        (if (>= i 0)
+          (begin
+            (vector-set! x i (expr->source (vector-ref v i)))
+            (loop (- i 1)))))
+      x))
+
+  (expr->source expr))
+
+;; (source->expression source) returns the Scheme expression
+;; represented by the source 'source'.  Note that every call with the
+;; same argument returns a different (i.e. non eq?) expression.
+
+(define (source->expression source)
+
+  (define (list->expression l)
+    (cond ((pair? l)
+           (cons (source->expression (car l)) (list->expression (cdr l))))
+          ((null? l)
+           '())
+          (else
+           (source->expression l))))
+
+  (define (vector->expression v)
+    (let* ((len (vector-length v))
+           (x (make-vector len)))
+      (let loop ((i (- len 1)))
+        (if (>= i 0)
+          (begin
+            (vector-set! x i (source->expression (vector-ref v i)))
+            (loop (- i 1)))))
+      x))
+
+  (let ((code (source-code source)))
+    (cond ((pair? code)
+           (list->expression code))
+          ((box-object? code)
+           (box-object (source->expression (unbox-object code))))
+          ((vector-object? code)
+           (vector->expression code))
+          (else
+           code))))
+
+;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+
+;; (include-expr->source source info-port) returns the source
+;; representation of a "begin" form containing the expressions
+;; contained in the specified file.
+
+(define (include-expr->source source info-port)
+
+  (define (find-source-file filename)
+
+    (define (open-error filename)
+      (pt-syntax-error
+       source
+       "Can't find file"
+       (or (path-expand filename)
+           filename)))
+
+    (let ((expanded-filename (path-expand filename)))
+      (if expanded-filename
+        (if (equal? (path-extension expanded-filename) "")
+
+          (let loop ((exts scm-file-exts))
+            (if (pair? exts)
+              (let* ((ext (car exts))
+                     (full-name (string-append expanded-filename ext))
+                     (port (open-input-file* full-name)))
+                (if port
+                  (begin
+                    (close-input-port port)
+                    full-name)
+                  (loop (cdr exts))))
+              (open-error filename)))
+
+          (let ((port (open-input-file* expanded-filename)))
+            (if port
+              (begin
+                (close-input-port port)
+                expanded-filename)
+              (open-error filename))))
+
+        (open-error filename))))
+
+  (let* ((filename-src
+          (cadr (source-code source)))
+         (filename
+          (source-code filename-src))
+         (rerooted-filename
+          (path-expand
+           filename
+           (path-directory (path-expand (locat-filename (source-locat filename-src))))))
+         (final-filename
+          (find-source-file rerooted-filename))
+         (re
+          (**readenv-open final-filename)))
+
+    (define (read-sources) ; return list of all sources in file
+';;;;;;;;;;;;;;;;;;;;;;;;;;;
+      (let ((source ((or read-datum-or-eof **read-datum-or-eof) re)))
+        (if (vector-object? source)
+          (begin
+            (if info-port (display "." info-port))
+            (cons source (read-sources)))
+          '()))
+      (##vector-ref
+       (##read-all-as-a-begin-expr-from-port
+        (**readenv-port re)
+        (##current-readtable)
+        (lambda (re x)
+          (make-source x (##make-locat-from-readenv re)))
+        (lambda (re x)
+          (source-code x))
+        #f
+        #f)
+       1));;;;;;;;;;;;;;;;;;;;;;;
+
+    (if info-port
+      (begin
+        (display "(reading " info-port)
+        (write (path-expand final-filename) info-port)))
+
+    (let ((sources (read-sources)))
+
+      (if info-port (display ")" info-port))
+
+      (**readenv-close re)
+
+      sources)))
+)
