@@ -1,7 +1,7 @@
 ;;; -*- Mode:Emacs-Lisp -*-
 ;;; gambit.el --- Run Gambit in an [X]Emacs buffer
 
-;; Copyright (c) 1997-1998 Marc Feeley & Michael Sperber
+;; Copyright (c) 1997-2004 Marc Feeley & Michael Sperber
 
 ;; Authors: Marc Feeley <feeley@iro.umontreal.ca>
 ;;          Mike Sperber <sperber@informatik.uni-tuebingen.de>
@@ -39,11 +39,11 @@
 ;; "C-c _" can be used to delete the last popup window that was
 ;; created to highlight a Scheme expression.
 
-;------------------------------------------------------------------------------
+;;;----------------------------------------------------------------------------
 
 ;; User overridable parameters.
 
-(defvar scheme-program-name "gsi -:t") ; "-:t" needed for Windows95/NT
+(defvar scheme-program-name "gsi -:d-")
 
 (defvar gambit-repl-command-prefix "\C-c"
   "Emacs keybinding prefix for Gambit REPL's commands.")
@@ -71,7 +71,7 @@
 (defvar gambit-move-to-highlighted (not gambit-highlight-face)
   "Flag to move to window opened to highlight a Scheme expression.")
 
-;------------------------------------------------------------------------------
+;;;----------------------------------------------------------------------------
 
 ;; These must be loaded first because we redefine some of the
 ;; functions they contain.
@@ -79,7 +79,69 @@
 (require 'scheme)
 (require 'cmuscheme)
 
-;------------------------------------------------------------------------------
+;;;----------------------------------------------------------------------------
+
+(defun gambit-indent-function (indent-point state)
+  (let ((normal-indent (current-column)))
+    (goto-char (1+ (elt state 1)))
+    (parse-partial-sexp (point) calculate-lisp-indent-last-sexp 0 t)
+    (if (and (elt state 2)
+             (not (looking-at "\\sw\\|\\s_")))
+        ;; car of form doesn't seem to be a a symbol
+        (progn
+          (if (not (> (save-excursion (forward-line 1) (point))
+                      calculate-lisp-indent-last-sexp))
+              (progn (goto-char calculate-lisp-indent-last-sexp)
+                     (beginning-of-line)
+                     (parse-partial-sexp (point)
+                                         calculate-lisp-indent-last-sexp 0 t)))
+          ;; Indent under the list or under the first sexp on the same
+          ;; line as calculate-lisp-indent-last-sexp.  Note that first
+          ;; thing on that line has to be complete sexp since we are
+          ;; inside the innermost containing sexp.
+          (backward-prefix-chars)
+          (current-column))
+      (let ((function (buffer-substring (point)
+                                        (progn (forward-sexp 1) (point))))
+            method)
+        (setq method (or (gambit-indent-method function)
+                         (get (intern-soft function) 'scheme-indent-function)
+                         (get (intern-soft function) 'scheme-indent-hook)))
+        (cond ((or (eq method 'defun)
+                   (and (null method)
+                        (> (length function) 3)
+                        (string-match "\\`def" function)))
+               (lisp-indent-defform state indent-point))
+              ((integerp method)
+               (lisp-indent-specform method state
+                                     indent-point normal-indent))
+              (method
+                (funcall method state indent-point normal-indent)))))))
+
+(defun gambit-indent-method (function)
+  (let ((method nil)
+        (alist gambit-indent-regexp-alist))
+    (while (and (not method) (not (null alist)))
+      (let* ((regexp (car alist))
+             (x (string-match (car regexp) function)))
+        (if x
+            (setq method (cdr regexp)))
+        (setq alist (cdr alist))))
+    method))
+
+(set lisp-indent-function 'gambit-indent-function)
+
+(defvar gambit-indent-regexp-alist
+  '(
+    ("^declare$"               . defun)
+    ("^##declare$"             . defun)
+    ("^##define"               . defun)
+    ("^macro-check"            . defun)
+    ("^macro-force-vars$"      . defun)
+    ("^macro-number-dispatch$" . defun)
+   ))
+
+;;;----------------------------------------------------------------------------
 
 ;; Portable functions for FSF Emacs and Xemacs.
 
@@ -111,7 +173,7 @@
     (defun move-overlay (overlay start end buffer)
       (set-extent-endpoints overlay start end buffer)))
 
-;------------------------------------------------------------------------------
+;;;----------------------------------------------------------------------------
 
 ;; Redefine the function scheme-send-region from `cmuscheme' so
 ;; that we can keep track of all text sent to Gambit's stdin.
@@ -161,7 +223,31 @@
     (put-text-property 1 end 'rear-nonsticky '(read-only) buffer)
     (put-text-property 1 end 'read-only t buffer)))
 
-;------------------------------------------------------------------------------
+;;;----------------------------------------------------------------------------
+
+(defun gambit-load-file (file-name)
+  "Load a Scheme file FILE-NAME into the inferior Scheme process."
+  (interactive (comint-get-source "Load Scheme file: " scheme-prev-l/c-dir/file
+                                  scheme-source-modes t)) ; T because LOAD
+                                                          ; needs an exact name
+  (comint-check-source file-name) ; Check to see if buffer needs saved.
+  (setq scheme-prev-l/c-dir/file (cons (file-name-directory    file-name)
+                                       (file-name-nondirectory file-name)))
+  (scheme-send-string (concat "(load \"" file-name "\"\)\n")))
+
+(defun gambit-compile-file (file-name)
+  "Compile a Scheme file FILE-NAME in the inferior Scheme process."
+  (interactive (comint-get-source "Compile Scheme file: "
+                                  scheme-prev-l/c-dir/file
+                                  scheme-source-modes
+                                  nil)) ; NIL because COMPILE doesn't
+                                        ; need an exact name.
+  (comint-check-source file-name) ; Check to see if buffer needs saved.
+  (setq scheme-prev-l/c-dir/file (cons (file-name-directory    file-name)
+                                       (file-name-nondirectory file-name)))
+  (scheme-send-string (concat "(compile-file \"" file-name "\"\)\n")))
+
+;;;----------------------------------------------------------------------------
 
 ;; Buffer local variables of the Gambit inferior process(es).
 
@@ -174,7 +260,7 @@
 (defvar gambit-last-output-marker nil
   "Points to the last character output by the Gambit process.")
 
-;------------------------------------------------------------------------------
+;;;----------------------------------------------------------------------------
 
 ;; Utilities
 
@@ -195,29 +281,37 @@
         (concat str "\n")
         str)))
 
-;------------------------------------------------------------------------------
+;;;----------------------------------------------------------------------------
 
 ;; Define keys for single stepping and continuation crawling.
 
-(defun gambit-continue ()
-  (interactive)
-  (scheme-send-string ",c"))
-
 (defun gambit-step-continuation ()
   (interactive)
-  (scheme-send-string ",s"))
+  (scheme-send-string "#||#,s;"))
 
 (defun gambit-leap-continuation ()
   (interactive)
-  (scheme-send-string ",l"))
+  (scheme-send-string "#||#,l;"))
+
+(defun gambit-continue ()
+  (interactive)
+  (scheme-send-string "#||#,c;"))
+
+(defun gambit-environment ()
+  (interactive)
+  (scheme-send-string "#||#,e;"))
+
+(defun gambit-backtrace ()
+  (interactive)
+  (scheme-send-string "#||#,b;"))
 
 (defun gambit-crawl-backtrace-newer ()
   (interactive)
-  (scheme-send-string ",-"))
+  (scheme-send-string "#||#,-;"))
   
 (defun gambit-crawl-backtrace-older ()
   (interactive)
-  (scheme-send-string ",+"))
+  (scheme-send-string "#||#,+;"))
 
 (defun gambit-kill-last-popup ()
   (interactive)
@@ -246,7 +340,7 @@
 
 (defvar gambit-popups nil)
 
-;------------------------------------------------------------------------------
+;;;----------------------------------------------------------------------------
 
 ;; Procedures to intercept and process the location information output
 ;; by Gambit.
@@ -313,6 +407,7 @@
 
 (defvar gambit-location-regexp-alist
   '(("\\(\\\"\\(\\\\\\\\\\|\\\\\"\\|[^\\\"\n]\\)+\\\"\\)@\\([0-9]+\\)\\.\\([0-9]+\\)[^0-9]" 1 3 4)
+    ("\\((console)\\)@\\([0-9]+\\)\\.\\([0-9]+\\)[^0-9]" 1 2 3)
     ("\\((stdin)\\)@\\([0-9]+\\)\\.\\([0-9]+\\)[^0-9]" 1 2 3)))
 
 (defun gambit-closest-non-following (line alist)
@@ -333,7 +428,8 @@
   (let ((name (car locat))
         (line (car (cdr locat)))
         (column (car (cdr (cdr locat)))))
-    (cond ((equal name '(stdin))
+    (cond ((or (equal name '(console))
+               (equal name '(stdin)))
            (let ((closest
                   (gambit-closest-non-following
                    line
@@ -465,7 +561,7 @@ enlarge the window if it is too small."
 (defun gambit-unhighlight ()
   (gambit-highlight-region (get-buffer scheme-buffer) 1 1))
 
-;------------------------------------------------------------------------------
+;;;----------------------------------------------------------------------------
 
 (defun gambit-install-comment-syntax ()
   "Configure #| ... |# comments."
@@ -500,12 +596,21 @@ enlarge the window if it is too small."
       (modify-syntax-entry ?| "\" 23" scheme-mode-syntax-table))))
 
 (defun gambit-extend-mode-map (map)
+  (define-key map [(f8)]  'gambit-continue)
+  (define-key map [(f9)]  'gambit-crawl-backtrace-newer)
+  (define-key map [(f10)] 'gambit-crawl-backtrace-older)
+  (define-key map [(f11)] 'gambit-step-continuation)
+  (define-key map [(f12)] 'gambit-leap-continuation)
+
+  (define-key map "\C-c\C-l" 'gambit-load-file)
+  (define-key map "\C-c\C-k" 'gambit-compile-file)
+
   (let ((prefix gambit-repl-command-prefix))
     (define-key map (concat prefix "c") 'gambit-continue)
+    (define-key map (concat prefix "]") 'gambit-crawl-backtrace-newer)
+    (define-key map (concat prefix "[") 'gambit-crawl-backtrace-older)
     (define-key map (concat prefix "s") 'gambit-step-continuation)
     (define-key map (concat prefix "l") 'gambit-leap-continuation)
-    (define-key map (concat prefix "[") 'gambit-crawl-backtrace-older)
-    (define-key map (concat prefix "]") 'gambit-crawl-backtrace-newer)
     (define-key map (concat prefix "_") 'gambit-kill-last-popup)))
 
 (defun gambit-inferior-mode ()
@@ -541,4 +646,4 @@ enlarge the window if it is too small."
 
 (provide 'gambit)
 
-;------------------------------------------------------------------------------
+;;;----------------------------------------------------------------------------
