@@ -15,11 +15,12 @@
   (run-benchmark
     "divrec"
     divrec-iters
-    (lambda () (recursive-div2 *ll*))
     (lambda (result)
       (equal? result
               '(() () () () () () () () () () () () () () () () () () () ()
                 () () () () () () () () () () () () () () () () () () () ()
                 () () () () () () () () () () () () () () () () () () () ()
                 () () () () () () () () () () () () () () () () () () () ()
-                () () () () () () () () () () () () () () () () () () () ())))))
+                () () () () () () () () () () () () () () () () () () () ())))
+    (lambda (l) (lambda () (recursive-div2 l)))
+    *ll*))
